
/******************************************************************************
* MODULE     : base.hpp
* DESCRIPTION: Basic inclusions, definitions and routines
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_BASE_HPP
#define __MML_BASE_HPP
#include <gc.h>
#include <stdlib.h>
#include <iostream>
#include "config.hpp"
using std::istream;
using std::ostream;
using std::cin;
using std::cout;
using std::cerr;
namespace mml {

#define INDIRECT_PROTO(T,R)                                             \
private:						                \
  R* rep;						                \
public:							                \
  inline T (R* rep2);              		                \
  inline T (const T& x);                 		                \
  inline ~T ();   					                \
  inline T& operator= (const T& x);                              	\
  inline R* operator-> () const;

#define INDIRECT_IMPL(T,R)                                              \
  inline T::T (R* rep2): rep (rep2) {}   		                \
  inline T::T (const T& x) : rep (x.rep) {}		                \
  inline T::~T () {}					                \
  inline T& T::operator= (const T& x) { rep= x.rep; return *this; }	\
  inline R* T::operator-> () const { return rep; }

inline void* operator new   (register size_t sz) { return GC_malloc (sz); }
inline void* operator new[] (register size_t sz) { return GC_malloc (sz); }
inline void operator delete (register void* ptr)   { (void) ptr; }
inline void operator delete[] (register void* ptr) { (void) ptr; }

volatile void fatal_error (char* message, char* routine, char* file);
void mml_init ();

}
#endif // __MML_BASE_HPP
